require 'omniauth-oauth2'

module OmniAuth
  module Strategies
    class Aliyun < OmniAuth::Strategies::OAuth2
      option :client_options, {
          :site => 'https://oauth.aliyun.com',
          :authorize_url => 'https://signin.aliyun.com/oauth2/v1/auth',
          :token_url => 'https://oauth.aliyun.com/v1/token',
      }

      def request_phase
        super
      end

      def authorize_params
        super.tap do |params|
          %w[scope client_options].each do |v|
            if request.params[v]
              params[v.to_sym] = request.params[v]
              # to support omniauth-oauth2's auto csrf protection
              session['omniauth.state'] = params[:state] if v == 'state'
            end
          end
        end
      end

      uid { raw_info['aid'].to_s }

      info do
        {
          'nickname' => raw_info['upn'] || raw_info['login_name'],
          'name' => raw_info['name'],
          'email' => raw_info['upn'] || raw_info['login_name'] # 阿里云登录返回信息不包含邮箱，这里的 email 获取并不准确
        }
      end

      extra do
        {:raw_info => raw_info}
      end

      def raw_info
	      access_token.options[:param_name] = 'access_token'
        access_token.options[:mode] = :query
        @raw_info ||= access_token.get('/v1/userinfo').parsed
      end
    end
  end
end

OmniAuth.config.add_camelization 'aliyun', 'Aliyun'
